package CheckRelease::ListProperty;

use strict;
use warnings;

use Data::Dumper;
use MT;
use MT::Entry;
use MT::Template;
use MT::ListProperty;
use MT::Util qw( trim remove_html );
use CheckReleaseEditor::Util;
use CheckReleaseWriter::Util;

use CheckRelease::CMS::Entry;
use constant SUBSCRIBED => CheckRelease::CMS::Entry::SUBSCRIBED();
use constant APPROVAL => CheckRelease::CMS::Entry::APPROVAL();
use constant RETURN => CheckRelease::CMS::Entry::RETURN();

use CheckRelease::Util qw( get_config_value get_config_hash is_editor is_administer load_latest_revision unable_plugin );

sub _hdlr_condition {
	my $app = MT->instance;
	my $user = $app->user;
	return 0 if unable_plugin();
	1;
}

sub _hdlr_entry_title_html {
	my $static_path = MT->config->StaticWebPath;
	$static_path .= '/' if $static_path !~ m!/$!;

	my $prop        = shift;
	my ($obj, $app)       = @_;

	if ( unable_plugin() || !ref( $app->blog ) ) {
		return MT::Entry->list_props()->{title}->{html}->($prop, @_);
	}

	my $user = $app->user;
	my $class       = $obj->class;
	my $class_label = $obj->class_label;
	my $title       = $prop->super(@_);
	my $excerpt     = remove_html( $obj->excerpt )
	    || remove_html( $obj->text );
	## FIXME: Hard coded
	my $len = 40;
	if ( length $excerpt > $len ) {
	    $excerpt = substr( $excerpt, 0, $len );
	    $excerpt .= '...';
	}
	my $id        = $obj->id;
	my $permalink = $obj->permalink;
	my $edit_url  = MT->app->uri(
	    mode => 'view',
	    args => {
	        _type   => $class,
	        id      => $obj->id,
	        blog_id => $obj->blog_id,
	    }
	);
	my $status = $obj->status;
	my $status_class
		= $status == MT::Entry::HOLD()      ? 'Draft'
		: $status == MT::Entry::RELEASE()   ? 'Published'
		: $status == MT::Entry::REVIEW()    ? 'Review'
		: $status == MT::Entry::FUTURE()    ? 'Future'
		: $status == MT::Entry::JUNK()      ? 'Junk'
		: $status == 6 						? 'Unpublish' # 6=MT::Entry::UNPUBLISH() compatibility under 6
		:                                     '';
	my $lc_status_class = lc $status_class;
	require MT::Entry;
	my $status_file
		= $status == MT::Entry::HOLD()      ? 'draft.gif'
		: $status == MT::Entry::RELEASE()   ? 'success.gif'
		: $status == MT::Entry::REVIEW()    ? 'warning.gif'
		: $status == MT::Entry::FUTURE()    ? 'future.gif'
		: $status == MT::Entry::JUNK()      ? 'warning.gif'
		: $status == 6 						? 'unpublished.gif' # 6=MT::Entry::UNPUBLISH() compatibility under 6
		:                                     '';

	my $status_img
	    = $static_path . 'images/status_icons/' . $status_file;
	my $view_img
	    = $static_path . 'images/status_icons/view.gif';

	# check check_status
	my $plugin = MT->component('CheckRelease');
	my $check_status = $obj->check_status || 0;
	my $is_approved = $obj->status == MT::Entry::RELEASE()
		|| $obj->status == MT::Entry::FUTURE()
		|| $obj->status == MT::Entry::JUNK()
		|| $obj->status == 6 ? 1 : 0; # 6=MT::Entry::UNPUBLISH() compatibility under 6
	if ( !$is_approved ) {
		if ( $check_status == SUBSCRIBED() ) {
			$status_img = $static_path . 'images/status_icons/warning.gif';
			$status_class = $plugin->translate('Subscribing');
		} elsif ( $check_status == RETURN() ) {
			$status_img = $static_path . 'images/status_icons/share.gif';
			$status_class = $plugin->translate('ReturningTo');
		}
	}

	my $view_link = $obj->status == MT::Entry::RELEASE()
	    ? qq{
	    <span class="view-link">
	      <a href="$permalink" target="_blank">
	        <img alt="View $class_label" src="$view_img" />
	      </a>
	    </span>
	}
	    : '';

	my $scope = 'blog:' .$app->param('blog_id');
	my $cross_check = is_administer($user, $obj->blog_id) ? get_config_value('checker_cross_check', $scope) : undef;
	my $lock_entry = get_config_value('lock_entry', $scope);

	# judge link or not
	my $cond = 0;
	if ( is_administer($user, $obj->blog_id) ) {
		++$cond if $cross_check && $lock_entry && $obj->created_by == $user->id && $check_status == SUBSCRIBED();
	} else {
		++$cond if $lock_entry && $obj->created_by == $user->id && $check_status == SUBSCRIBED();
		++$cond if $obj->status == MT::Entry::RELEASE() || $obj->status == MT::Entry::FUTURE();
	}
	$cond = 0 if $obj->class ne 'entry';
	my $out = '';

	# except <a>
	($title) = ($title =~ m!<a[^>]+?>(.+?)</a>!ism) if $cond;

	$out = ( $cond )
		? qq{
	    <span class="icon status $lc_status_class">
	      <img alt="$status_class" src="$status_img" />
	    </span>
	    <span class="title">
	      $title
	    </span>
	    $view_link
		}
		: qq{
	    <span class="icon status $lc_status_class">
	      <a href="$edit_url"><img alt="$status_class" src="$status_img" /></a>
	    </span>
	    <span class="title">
	      $title
	    </span>
	    $view_link
	};
	$out .= qq{<p class="excerpt description">$excerpt</p>}
	    if trim($excerpt);
	return $out;
}

sub _hdlr_entry_revision_html {
	my ($prop, $obj, $app, $condition) = @_;
	return '' if unable_plugin();
	return '' unless ref $app->blog;

	my $config = get_config_hash();
	my $user = $app->user;
	my $class = $obj->class;


	# 記事ロックの設定を取得
	my $lock_entry = ( defined($config->{lock_entry})
		&& $config->{lock_entry} ) ? 1 : 0;

	# 承認者同士の承認設定を取得
	my $cross_check = ( defined($config->{checker_cross_check})
		&& $config->{checker_cross_check} ) ? 1 : 0;

	my $cols = $obj->column_names;

	my $blog_id = $app->param('blog_id');
	my $perms = $user->blog_perm($blog_id);
	my $can_edit_all_posts = $perms->can_edit_all_posts() || 0;
	my $can_publish_post = $perms->can_publish_post || 0;

	my $entry = {};
	$entry = is_administer($user, $blog_id)
		? CheckReleaseEditor::CMS::Entry::get_entry_status(
			$app, $user, $entry, $class, $lock_entry, $cross_check, $obj)
		: CheckReleaseWriter::CMS::Entry::get_entry_status(
			$app, $user, $entry, $class, $lock_entry, $can_edit_all_posts, $can_publish_post, $obj);

	list_html($app, $entry, $app->blog);
}

sub list_html {
	my ($app, $entry, $blog) = @_;
	return '' unless ref $entry;

	my $user = $app->user;
	my $tmpl = MT::Template->new;
	$tmpl->blog_id( $blog->id );
	$tmpl->text(
		is_administer($user, $blog->id)
			? CheckReleaseEditor::Util::list_html_tmpl()
			: CheckReleaseWriter::Util::list_html_tmpl()
		);
	my $ctx = $tmpl->context;
	$ctx->stash('blog', $blog);
	my $vars = $ctx->stash('vars') || {};
	foreach ( keys %$entry ) {
		$vars->{$_} = $entry->{$_};
	}
	$vars->{_status_Draft} = 1;
	$vars->{object_type} = $entry->{class};
	$vars->{original_blog_id} = $entry->{blog_id};
	$vars->{static_uri} = $app->static_path;
	$vars->{script_url} = $app->mt_uri;
	$ctx->stash('vars', $vars);

	my $html = $tmpl->build($ctx);
	$html;
}

1;
