package CheckRelease::Task;

# プラグマ
use strict;

# モジュール
use File::Spec;
use MT;
use MT::Util;
use Data::Dumper;
use CustomFields::Util qw(save_meta);
use CheckRelease::Util qw(unable_plugin);

# 指定日公開処理
sub future_post {
	my $task = shift;
	return 1 if unable_plugin();
	
	# キューを取得
	my $key = 'futurepost_que';
	my $name = 'CheckRelease::Plugin';
	
	require MT::PluginData;
	my $terms;
	$terms->{key} = $key;
	$terms->{plugin} = $name;
	my $pd = MT::PluginData->load( $terms );
	return 1 unless ref $pd;
	
	my $data = $pd->data();
	return 1 unless %$data;
	
	# リビジョンと記事IDを取得
	my @rev_ids;
	my @entry_ids;
	foreach ( %$data ) {
		push @entry_ids, $_;
		push @rev_ids, $data->{$_};
	}
	
	# 記事の呼び出し
	my $pkg = 'MT::Entry';
	my $term;
	$term->{id} = \@entry_ids;
	my $count = $pkg->count( $term );
	
	# 記事がなければ終了
	unless ( $count ) {
		$pd->data( {} );
		$pd->save
			or die $pd->errstr;
		return 1;
	}
	
	my $iter = $pkg->load_iter( $term );
	
	# リビジョンの呼び出し
	my $rev_pkg = 'MT::Entry::Revision';
	my $t;
	$t->{id} = [ grep { defined($_) } map { $_ } @rev_ids ];
	my @revs = $rev_pkg->load($t);
	my %revs = map { $_->id => $_ } @revs;
	
	# 置き換えと再構築
	require MT::Serialize;
	require MT::Author;
	require MT::Tag;
	require MT::Placement;
	my %authors;
	my %blogs;
	my $cols = [];
	while ( my $entry = $iter->() ) {
		my $clone = $entry->clone;
		
		$cols = $entry->column_names unless scalar @$cols;
		
		# あとでインデックステンプレートを再構築するためにキューに貯める
		my $blog = $entry->blog;
		$blogs{ $blog->id } = $blog unless defined $blogs{ $blog->id };
		
		my $rev_id = $data->{ $entry->id };
		my $rev_obj = defined($revs{ $rev_id }) ? $revs{ $rev_id } : '';
		
		# リビジョンが存在しなければ次へ
		unless ( ref($rev_obj) ) {
			delete $data->{ $entry->id };
			next;
		}
		
		# リビジョンのデーターを復元
		my $packed = MT::Serialize->unserialize( $rev_obj->entry );
		unless ( %$$packed ) {
			delete $data->{ $entry->id };
			next;
		}
		
		# 時間を比較して処理するかどうか検討
        my @ts = MT::Util::offset_time_list( time, $blog );
        my $now = sprintf "%04d%02d%02d%02d%02d%02d", $ts[5] + 1900, $ts[4] + 1,
          @ts[ 3, 2, 1, 0 ];
		next unless $now > $$packed->{authored_on};
		next unless $$packed->{status} == MT::Entry::FUTURE();
		
		# 記事の内容をリビジョンの内容で上書き
		foreach ( @$cols ) {
			$entry->$_( $$packed->{$_} );
		}
		$entry->status( MT::Entry::RELEASE() );
		
		# カスタムフィールドの値を上書き
		my $meta = $entry->meta;
		foreach ( keys %$$packed ) {
			if ( /^field\./ ) {
				$meta->{$_} = defined($$packed->{$_}) ? $$packed->{$_} : undef;
			}
		}
		
		# タグの上書き
		my $author_id = $rev_obj->modified_by || $rev_obj->created_by;
		my $user = defined($authors{ $author_id })
			? $authors{ $author_id } : undef;
		my $delim = ',';
		unless ( defined $user ) {
			$user = MT::Author->load({ id => $author_id });
			$authors{ $author_id } = $user if ref $user;
			$delim = $user->entry_prefs->{tag_delim} if ref $user;
		}
		
		my $tags = $$packed->{tags} || '';
		my @tags = MT::Tag->split($delim, $tags);
		
		# カテゴリ情報の取得
		my $places = $$packed->{placement};
		my $orig_pls;
		my $orig_pls_hash = [];
		my $new_pls = [];
		my $pl_cols = [];
		if ( @$places ) {
			my @pls = MT::Placement->load( { entry_id => $entry->id } );
			$orig_pls = \@pls if \@pls;
			# 再構築失敗時の復元のためハッシュ化して一時保存
			#$orig_pls_hash = to_hash( $orig_pls );
			foreach my $pl_hash ( @$places ) {
				my $p = MT::Placement->new;
				$pl_cols = $p->column_names unless @$pl_cols;
				$p->$_( $pl_hash->{$_} ) foreach @$pl_cols;
				push @$new_pls, $p;
			}
		}
		
		# アイテム情報の取得
		my $object_asset = $$packed->{objectasset};
		my $orig_obj_asset;
		my $orig_obj_asset_hash = [];
		my $new_obj_asset = [];
		my $oa_cols = [];
		if ( @$object_asset ) {
			my $oa_terms;
			$oa_terms->{object_ds} = 'entry';
			$oa_terms->{object_id} = $entry->id;
			my @oa = MT::ObjectAsset->load( $terms );
			$orig_obj_asset = \@oa if @oa;
			# 再構築失敗時の復元のためハッシュ化して一時保存
			#$orig_obj_asset_hash = to_hash( $orig_pls );
			
			foreach my $oa_pack ( @$object_asset ) {
				my $oa = MT::ObjectAsset->new;
				$oa_cols = $oa->column_names unless @$oa_cols;
				$oa->$_( $oa_pack->{$_} ) foreach @$oa_cols;
				push @$new_obj_asset, $oa;
			}
		}
		
		# タグの上書き
		if ( @tags ) {
			$entry->set_tags( @tags );
		} else {
			$entry->remove_tags();
		}
		$entry->meta( $meta );
		
		my $latest_rev = load_latest_revision($entry->datasource, $entry->id);
		$entry->current_revision( $latest_rev->rev_number ) if ref $latest_rev;
		
		# ブログ記事オブジェクトの保存
		$entry->save()
			or next;
		save_meta( $entry, $meta );
			
		# カテゴリの更新
		$_->remove() foreach @$orig_pls;
		$_->save() foreach @$new_pls;
		
		# アイテムの更新
		my @oa_clones = map { $_->clone } @$orig_obj_asset;
		$_->remove() foreach @$orig_obj_asset;
		$_->save() foreach @$new_obj_asset;
		
		# 再構築
		eval {
			MT->rebuild_entry( Entry => $entry, Blog => $blog )
				or die MT->errstr;
		};
		if ( $@ ) {
            require MT::Log;
            MT->log(
                {
                    message => MT->translate(
"An error occurred while publishing scheduled entries: [_1]",
                        $@
                    ),
                    class   => "system",
                    blog_id => $blog->id,
                    level   => MT::Log::ERROR()
                }
            );
            
			# カテゴリ情報の復元
			#$_->remove() foreach @$new_pls;
			#if ( @$orig_pls_hash ) {
			#	foreach my $pl_hash ( @$orig_pls_hash ) {
			#		my $pl_obj = MT::Placement->new();
			#		foreach ( @$pl_cols ) {
			#			$pl_obj->$_( $pl_hash->{$_} ) unless $_ eq 'id';
			#		}
			#		$pl_obj->save();
			#	}
			#}
			
			# アイテム情報の復元
			#$_->remove() foreach @$new_obj_asset;
			#if ( @$orig_obj_asset_hash ) {
			#	foreach my $oa_hash ( @$orig_obj_asset_hash ) {
			#		my $oa_obj = MT::ObjectAsset->new;
			#		foreach ( @$oa_cols ) {
			#			$oa_obj->$_( $oa_hash->{$_} );
			#		}
			#		$oa_obj->save();
			#	}
			#}
            next;
		}
		
		# 消しこみ
		delete $data->{ $entry->id };
	}
	
	# インデックステンプレートを再構築
	foreach ( %blogs ) {
		MT->rebuild_indexes( Blog => $blogs{$_} );
	}
	
	# キューの上書き
	$pd->data( $data );
	$pd->save();
	1;
}

sub to_hash {
	my ($objs) = @_;
	my $cols = [];
	return [] unless @$objs;
	my $hashes = [];
	foreach my $obj ( @$objs ) {
		$cols = $obj->column_names unless @$cols;
		my %row = map { $_ => $obj->$_ } grep { $_ ne 'id' } @$cols;
		push @$hashes, \%row;
	}
	$hashes;
}

sub load_latest_revision {
	my ($class, $id) = @_;
	my $pkg = 'MT::Entry::Revision';
	return undef unless $pkg;
	my $terms;
	$terms->{$class .'_id'} = $id;
	my $args;
	$args->{limit} = 1;
	$args->{'sort'} = 'rev_number';
	$args->{direction} = 'descend';
	my @revs = $pkg->load($terms, $args);
	@revs ? shift @revs : undef;
}


1;