package CheckReleaseEditor::CMS::Entry;

use strict;
use warnings;

use MT;
use base qw(CheckRelease::CMS::Entry);

use MT::App::CMS;
use MT::Serialize;
use MT::Util qw(epoch2ts format_ts is_valid_email);
use CheckReleaseEditor::Util qw( list_html_tmpl );
use Ideamans::Util qw(array_search);
use Data::Dumper;

use CheckRelease::Util qw( get_permissions get_config_value get_config_hash
		check_url _scope want_cross_check get_checkers is_editor button_html mail_head
		update_future_que load_latest_revision send_notice_mail cf_meta load_checkers
		get_crChangeBtnLabel unable_plugin);

use constant SUBSCRIBED => CheckRelease::CMS::Entry::SUBSCRIBED();
use constant APPROVAL => CheckRelease::CMS::Entry::APPROVAL();
use constant RETURN => CheckRelease::CMS::Entry::RETURN();

sub _hdlr_template_source_edit_entry {
	my ($eh, $app, $tmpl) = @_;
	my $q = $app->param;
	return unless $q->param('_type') eq 'entry';
	return $$tmpl if unable_plugin();

	my $user = $app->user;

	# 承認者でなければ処理不要
	return unless is_editor($user);

	# 承認者同士のチェックが必要かどうか
	my $cross_check = get_config_value('checker_cross_check');

	my $entry_id = $app->param('id') || '';
	my $draft_button_text = '';
	my $save_draft = '';
	my $draft_button_html = '';

	if ( $cross_check ) {
		if ( MT->version_number < 5.1 ) {
			# 承認必須でも下書きできるように「下書き」ボタンの追加
			my $blog_id = $app->param('blog_id');
			my $config = get_config_hash("blog:$blog_id");
			my $obj = $app->model('entry')->load($entry_id);
			if ( !($config->{select_checker} && !$config->{check_required})
				&& (!defined($obj) || $obj->author_id == $user->id) ) {
				my $old = q{(<div class="actions-bar.+?<button.+?class="preview.+?<\/div>)};
				my $save_draft_btn = q{<div class="actions-bar save-action" style="width:100%; clear:both;float:none;margin-top:12px;">
          <button
             name="save_draft"
             type="submit"
             title="<__trans phrase="Draft" />"
             class="preview draft action button btn-save-draft"
             onClick="javascript:jQuery('#save_draft').val(1);"
             mt:mode="save_entry"
             value="1"
             >
            <__trans phrase="Draft" />
          </button>
          <br />
          <input type="hidden" name="save_draft" id="save_draft" value="0" />
        </div>
				};
				$$tmpl =~ s!$old!$1$save_draft_btn!gms;
			}

			# 「下書き保存」の変更
			$draft_button_text = '<mt:var\s*name="draft_button_text">';
			$save_draft = '<__trans\s*phrase="Save Draft">';
			$draft_button_html = q{<span id="cr_draft_button_text"><mt:setvarblock name="label_checked"><mt:unless name="select_checker">1</mt:unless><mt:if name="check_required">1</mt:if></mt:setvarblock><__trans_section component="CheckRelease"><mt:if name="label_checked"><__trans phrase="writer_save_text_checked"><mt:else><__trans phrase="writer_save_text"></mt:if></__trans_section></span><script type="text/javascript">jQuery("#cr_draft_button_text").parent("button").each(function()\{jQuery(this).attr("title", "<__trans_section component="CheckRelease"><mt:if name="label_checked"><__trans phrase="writer_save_text_checked"><mt:else><__trans phrase="writer_save_text"></mt:if></__trans_section>");\});</script>};

			# 新規記事作成の場合
			unless ( $entry_id ) {
				$$tmpl =~ s!$draft_button_text!$draft_button_html!msg;
				$$tmpl =~ s!$save_draft!$draft_button_html!msg;
				return $$tmpl;
			}
		}
		else {
			# MT5.1 対応
			# 承認必須でも下書きできるように「下書き」ボタンの追加
			my $blog_id = $app->param('blog_id');
			my $config = get_config_hash("blog:$blog_id");
			my $crr = $app->param('crr') || undef;
			my $rev_obj = load_latest_revision($app, $app->param('_type'), $entry_id);
			my $obj = $app->model('entry')->load($entry_id);
			if (
				(!$config->{select_checker}
				|| ($config->{select_checker} && $config->{check_required})
				&& (!defined($obj) || $obj->author_id == $user->id) )
				&&
				(!defined($crr)
					|| ($app->param('r') == 0 || (defined($crr) && $rev_obj->created_by == $user->id))
				) ) {
				my $old = q{(<div class="preview\-action">.+?<\/div>)};
				my $save_draft_btn = q{<div class="save-action" style="width:100%;clear:both;float:none;padding-top:12px;">
          <button
             id="btn-save-draft"
             name="save_draft"
             type="submit"
             title="<__trans phrase="Draft" />"
             class="action button btn-save-draft"
             onClick="javascript:jQuery('#save_draft').val(1);">
            <__trans phrase="Draft" />
          </button>
          <script type="text/javascript">
            jQuery(document).ready(function() {
                function crShowHideBtnSaveDraft() {
                  var selected_val = '';
                  jQuery('select#status option:selected').each(function() {
                      selected_val = jQuery(this).attr('value');
                  });
                  if (selected_val == 1) {
                    jQuery('#btn-save-draft').show();
                  } else {
                    jQuery('#btn-save-draft').hide();
                  }

                }
                 crShowHideBtnSaveDraft();
                jQuery('select#status').change(function() {
                    crShowHideBtnSaveDraft();
                });
            });
          </script>
        </div>
        };
				$$tmpl =~ s!$old!$1$save_draft_btn!gms;
			}

			# 「保存」の変更
			my $button_text = '<mt:var\s*name="button_text">';
			$draft_button_html = q{<span id="cr_draft_button_text"><mt:setvarblock name="label_checked"><mt:unless name="select_checker">1</mt:unless><mt:if name="check_required">1</mt:if></mt:setvarblock><__trans_section component="CheckRelease"><mt:if name="label_checked"><mt:if name="created_by_is_me"><__trans phrase="writer_save_text_checked"><mt:else><__trans phrase="writer_save_text"></mt:if><mt:else><__trans phrase="writer_save_text"></mt:if></__trans_section></span>};
			my $draft_button_js = q{<script type="text/javascript">jQuery("#cr_draft_button_text").parent("button").each(function()\{jQuery(this).attr("title", "<__trans_section component="CheckRelease"><mt:if name="label_checked"><__trans phrase="writer_save_text_checked"><mt:else><__trans phrase="writer_save_text"></mt:if></__trans_section>");\});</script>};
			$$tmpl =~ s!$button_text!$draft_button_html$draft_button_js!gms;

      $draft_button_text = q{jQuery\('button\.publish'\)\.text\('<mt:var\s*name="draft_button_text">'\);};
      my $draft_button_replace = "jQuery('button.publish').html('" .$draft_button_html ."');";
      $$tmpl =~ s!$draft_button_text!$draft_button_replace!gms;

			my $set_save_js = q{setSaveButtonText\(\);};
			my $cr_change_btn_label = q{crChangeBtnLabel();};
			$cr_change_btn_label .= get_crChangeBtnLabel('editor');
			$$tmpl =~ s!$set_save_js!$cr_change_btn_label!gms;

			# 公開ステータス「未公開（原稿）」のときに承認者選択状態からボタンのラベルを調整する
			my $insert_hook = '        } else if \(selectedStatus == 4\) {';
			my $draft_status_button_js =<<'EOD';
	        } else if (<mt:var name="select_checker" /> && selectedStatus == 1 && checked_any_authorities()) {<__trans_section component="CheckRelease">
	            jQuery('button.publish').html('<span id="cr_draft_button_text"><mt:if name="created_by_is_me"><__trans phrase="writer_save_text_checked" /><mt:else><__trans phrase="writer_save_text" /></mt:if></span>');
	            jQuery('button.publish').attr('title', '<mt:if name="created_by_is_me"><__trans phrase="writer_save_text_checked" /><mt:else><__trans phrase="writer_save_text" /></mt:if>');</__trans_section>
EOD
			$$tmpl =~ s!($insert_hook)!$draft_status_button_js$1!mgs;
		}
	}

	# 差し戻し状態のhiddenの追加
	my $status_old = '';
	if ( MT->version_number < 5.1 ) {
		$status_old = '<input type="hidden" name="status" id="status"';
	}
	else {
		$status_old = '<div class="status">';
	}
	my $status_new = q{
<input type="hidden" name="save_draft" id="save_draft" value="0" />
<input type="hidden" name="check_status" id="check_status" value="<mt:var name="check_status" default="0" />" />
<input type="hidden" name="check_requested_authors" id="check_requested_authors" value="<mt:var name="check_requested_authors" />" />
<mt:if name="crr"><input type="hidden" name="crr" id="crr" value="<mt:var name="crr" />" /></mt:if>
<input type="hidden" name="r" id="r" value="<mt:var name="r" />" />
};
	$$tmpl =~ s!$status_old!$status_new\n$status_old!msi;

	# 公開ボタンにonclick属性を追加
	if ( $cross_check ) {
		my $publish_old = '';
		my $publish_new = q{ onclick="javascript:crChackStatus();"};
		if ( MT->version_number < 5.1 ) {
			$publish_old = q{class="save draft action"};
		} else {
			$publish_old = q{class="publish action primary button"};
		}
		$$tmpl =~ s!($publish_old)!$1$publish_new!gms;
	}

	# リビジョンの新規作成の場合
	if ( $app->param('crr') ) {
		if ( ( defined($app->param('pre_r')) && $app->param('pre_r') == 0 )
			|| ( defined($app->param('r')) && $app->param('r') == 0 ) ) {

			my ($page_title_old,  $page_title_new);
			if ( MT->version_number < 5.1 ) {
				$page_title_old = '<\$mt:setvar name="page_title" value="<__trans phrase="Edit Entry">"\$>';

				# Save Draftの変換
				$$tmpl =~ s!$draft_button_text!$draft_button_html!msg
					if $cross_check;
			}
			else {
				$page_title_old = '<mt:setvar name="page_title" value="<__trans phrase="Edit Entry">">';
			}
			$page_title_new = q{<$mt:setvar name="page_title" value="<__trans_section component="CheckRelease"><__trans phrase="Edit Entry: New Revision"></__trans_section>"$>};

			# page_titleの修正
			$$tmpl =~ s!$page_title_old!$page_title_new!m;
		}

		# 公開ボタンのラベル修正
		my ($button_text_old, $button_text_new);
		if ( MT->version_number < 5.1 ) {
			$button_text_old = '<mt:setvar name="button_text" value="<__trans phrase="Publish">">';
			$button_text_new = q{<mt:setvar name="button_text" value="<__trans_section component="CheckRelease"><__trans phrase="Replace"></__trans_section>">};
			$$tmpl =~ s!$button_text_old!$button_text_new!msg;

			# 更新ボタンのラベル修正
			$button_text_old = '<mt:setvar name="button_text" value="<__trans phrase="Update">">';

			my $unpublish_button_old = '<__trans phrase="Unpublish">';
			my $unpublish_button_new = q{<span id="cr_draft_button_text"><__trans phrase="Unpublish"></span>};
			$$tmpl =~ s!$unpublish_button_old!$unpublish_button_new!msg;
		} else {
			$button_text_old = '<mt:setvar name="publish_button_text" value="<__trans phrase="Publish">">';
			$button_text_new = q{<mt:setvar name="publish_button_text" value="<__trans_section component="CheckRelease"><__trans phrase="Replace"></__trans_section>">};
		}
		$$tmpl =~ s!$button_text_old!$button_text_new!msg;
	}

	my $jQuery_old = "jQuery\\('input#title'\\)";
	if ( !$app->param('crr')
		|| ( $app->param('crr')
			&& ( $app->param('r') || $app->param('pre_r') )
			) ) {

		my $checkers = [];
		return $$tmpl unless $entry_id;

		my $created_by = '';
		if ( !$app->param('crr') ) {
			my $entry = $app->model('entry')->load($entry_id);
			return $$tmpl unless ref $entry;
			@$checkers = split /\s*,\s*/, $entry->check_requested_authors;
			$created_by = $entry->created_by;
		}
		else {
			my $rev_obj = load_latest_revision($app, 'entry', $entry_id);
			return $$tmpl unless ref $rev_obj;
			my $packed = $rev_obj->entry;
			$packed = MT::Serialize->unserialize($packed);
			@$checkers = split /\s*,\s*/, $$packed->{check_requested_authors};
			$created_by = $$packed->{created_by};
		}

		if ( array_search($user->id, $checkers) ) {
			if ( MT->version_number < 5.1 ) {
				# 保存ボタン
				my $old = '<button\s*mt:mode="save_entry"\s+?name="status"\s+?type="submit"\s+?title="<mt:var name="button_title">"\s+?class="publish action primary-button"\s+?value="2"\s+?><mt:var name="button_text"></button>';

				# 差し戻しボタンを表示
				my $new = '<mt:unless name="created_by_is_me">'.button_html('draft') .'</mt:unless>';

				# 差し戻しボタンを設置
				$$tmpl =~ s!($old)!$1$new!msi;
			}
			else {
				# MT5.1対応
				my $old = q{(<div\s*class="preview\-action">.+?<\/div>)};

				# 差戻しボタン
				my $new =<< 'EOT';
    <script type="text/javascript">
    function status_to_draft() {
      jQuery('#status').children('option').each(function() {
        if ( jQuery(this).attr('value') == 1 ) {
          jQuery(this).attr('selected', 'selected');
        } else {
          if ( jQuery(this).attr('selected') ) {
            jQuery(this).removeAttr('selected');
          }
        }
      });
    }
    </script>
    <div class="return-action" style="width:100%; clear:both; float:left; margin-top:12px;"><__trans_section component="CheckRelease">
      <button
         name="status"
         type="submit"
         title="<__trans phrase="Return to(Draft)" />"
         class="check_status action primary button" onclick="javascript:status_to_draft();">
        <__trans phrase="Return to(Draft)" />
      </button>
    </__trans_section></div>
EOT
          $$tmpl =~ s!$old!$1$new!msg;
			}

			my $jQuery_new = <<'EOS';
			jQuery('button.check_status').click(function(event) {
		        jQuery("input[name='status']").val(1);
		        jQuery("input[name='check_status']").val(3);
		    });
		    jQuery('button.publish').click(function(event) {
		        jQuery('input[name=status]').val(2);
		        jQuery('input[name=check_status]').val(0);
		    });
		    jQuery('button.draft').click(function(event) {
		        jQuery('input[name=status]').val(1);
		        jQuery('input[name=check_status]').val(0);
		    });
EOS

			$$tmpl =~ s!($jQuery_old)!$jQuery_new$1!msi;


			# プレビューボタン押下時のcheck_statusのPOST値を既存値に
			$jQuery_old = "jQuery\\('form#entry_form input\\[name=__mode\\]'\\).val\\('preview_entry'\\);";
			$jQuery_new = q{jQuery('input[name=check_status]').val(<mt:var name="check_status" encode_js="1" value="0" />);};
			$$tmpl =~ s!($jQuery_old)!$1\n$jQuery_new!m;
		}

		# ページロード時にボタンのラベルを切り替えるのJavaScriptの設置
		if ( $cross_check ) {
			if ( !$created_by || ($created_by && $user->id == $created_by) ) {
				my $onload_changebtnlabel = get_crChangeBtnLabel('editor');

				#$$tmpl =~ s!($jQuery_old)!$onload_changebtnlabel\n\n$1!ms;

				# 下書き保存ボタンのラベル
				$$tmpl =~ s!$draft_button_text!$draft_button_html!msg;
				$$tmpl =~ s!$save_draft!$draft_button_html!msg
					if $save_draft;
			}
		}
	}
	elsif ( MT->version_number >= 5.1 && $app->param('crr') && ( $app->param('r') == 0 || $app->param('pre_r') == 0 ) ) {
		my $jQuery_new = <<'EOS';
			jQuery('button.publish').click(function(evenet) {
				if ( jQuery('select#status').val() == 1 ) {
					jQuery('input[name=check_status]').val(0);
				}
			});
EOS
		$$tmpl =~ s!($jQuery_old)!$jQuery_new$1!msi;
	}

  # リビジョン保存後に表示される「ブログ記事を見る」を非表示
  if ($app->param('crr') && $app->param('r')) {
    if($app->param('saved_added') || $app->param('saved_changes')) {
      $$tmpl =~ s!<\$mt:var\sname="view_link"\$>!!g;
    }
  }

	$$tmpl;
}

sub template_param_to_display_cross_check_widget {
	my ($cb, $app, $param, $tmpl) = @_;
	my $q = $app->param;
	my $plugin = $app->component('CheckRelease');

	my $cls = $app->param('_type') || 'entry';
	my $user = $app->user || return;
	my $entry_id = $app->param('id') || return;
	my $entry = MT->model($cls)->load($entry_id) || return;
	my $rev = load_latest_revision($app, $cls, $entry_id);
	get_config_value('checker_cross_check') || return;

	my $hash = {};
	my $save_to;
	my @columns = ('author_id', 'status', 'check_status', 'check_requested_authors');
	if ( $app->param('crr') ) {
		$save_to = $rev;
		my $packed = MT::Serialize->unserialize($rev->$cls);
		foreach my $col ( @columns ) {
			$hash->{$col} = $$packed->{$col};
		}
	} else {
		$save_to = $entry;
		$hash->{_target} = 'original';
		foreach my $col ( @columns ) {
			$hash->{$col} = $entry->$col;
		}
	};

	return if $hash->{author_id} == $user->id;
	return if $hash->{status} != MT::Entry::HOLD();
	return if $hash->{check_status} != SUBSCRIBED();

	# 新規リビジョンでは非表示
	return if $app->param('crr')
		&& defined($app->param('r'))
		&& $app->param('r') eq '0';

	if ( get_config_value('select_checker') ) {
		my $selected = '';
		if ( defined($q->param('checkrelease_cross_checkers')) ) {
			my @ids = $q->param('checkrelease_cross_checkers');
			$selected = \@ids;
		}

		my @selected = ref($selected) ? @$selected : split(/\s*\s*/, $selected);
		$param->{cr_cross_checker_loop} = load_checkers(\@selected);

		my %indexed = map { $_ => 1 } split(/\s*,\s*/, $hash->{check_requested_authors} || '');
		foreach my $c ( @{$param->{cr_cross_checker_loop}} ) {
			$c->{current} = 1 if $indexed{$c->{id}};
		}
	}

	my $widget = $tmpl->createElement('app:widget', {
		id => 'checkrelease-cross-check',
		label => $plugin->translate('Transfer Approval'),
	});
	my $publishing_node = $tmpl->getElementById('entry-publishing-widget');

	# 承認者承認の承認者選択HTML
	my $innerHTML = <<'EOT';
	<__trans_section component="CheckRelease">
		<mt:if name="cr_cross_checker_loop">
			<p><__trans phrase="Send email to another checkers."></p>
			<mt:loop name="cr_cross_checker_loop">
			<label>
				<input type="checkbox" name="checkrelease_cross_checkers" class="cr_check_item cr_cross_checker" value="<mt:var name="id" />"<mt:if name="checked"> checked="checked"</mt:if> onClick="javascript:crUpdateCrossChecker();" />
				<mt:var name="label" />
				<mt:if name="current">
					<__trans phrase="(Checking)" />
				</mt:if>
			</label><br />
			</mt:loop>
			<script type="text/javascript">
				function crUpdateCrossChecker() {
					var $ = jQuery;
					var $status = $('#status'),
						$widget = $('#checkrelease-cross-check'),
						$button = $('#cross_check_button'),
						$selected = $('.cr_cross_checker:checked');

					$status.val() == 1 ? $widget.show() : $widget.hide();
					$selected.length > 0 ? $button.show() : $button.hide();
				}

				(function() {
					var $ = jQuery;

					$(function() {
						$('#status').change(crUpdateCrossChecker);
						crUpdateCrossChecker();
					});
				})();
			</script>
		<mt:else>
			<p><__trans phrase="Send email to all checkes again." /></p>
		</mt:if>
		<div id="cross_check_button" class="return-action" style="clear:both; margin-top:12px">
			<button name="cr_cross_check" value="1" type="submit" title="<__trans phrase='writer_save_text_checked'>" class="action primary button" style="width:100%;"><__trans phrase="writer_save_text_checked"></button>
		</div>

	</__trans_section>
EOT
	$widget->innerHTML( $innerHTML );
	$tmpl->insertAfter($widget, $publishing_node);

}

sub cms_post_save_to_send_cross_check_mail {
	my ($cb, $app, $obj, $obj_org) = @_;

	my $cross_check = $app->param('cr_cross_check');
	return unless $cross_check;

	my $cls = $app->param('_type') || 'entry';
	my $user = $app->user || return;
	my $entry_id = $app->param('id') || return;
	my $entry = MT->model($cls)->load($entry_id) || return;
	return unless $entry;
	my $rev = load_latest_revision($app, $cls, $entry_id);
	get_config_value('checker_cross_check') || return;

	my $hash = {};
	my $save_to;
	my @columns = ('author_id', 'status', 'check_status', 'check_requested_authors');
	if ( $app->param('crr') ) {
		$save_to = $rev;
		my $packed = MT::Serialize->unserialize($rev->$cls);
		foreach my $col ( @columns ) {
			$hash->{$col} = $$packed->{$col};
		}
	} else {
		$save_to = $entry;
		$hash->{_target} = 'original';
		foreach my $col ( @columns ) {
			$hash->{$col} = $entry->$col;
		}
	};

	return if $hash->{author_id} == $user->id;
	return if $hash->{status} != MT::Entry::HOLD();
	return if $hash->{check_status} != SUBSCRIBED();

	my $terms;
	if ( get_config_value('select_checker') ) {
		my $q = $app->param;
		my @selected = $q->param('checkrelease_cross_checkers');
		$terms = { id => \@selected };
	}

	my $checkers = get_checkers($terms);
	my @ids = map { $_->id } @$checkers;
	my $ids = join(',', @ids);

	if ( $ids ) {
		# 承認者の変更を保存
		if ( ref($save_to) =~ /Revision/ ) {
			my $packed = MT::Serialize->unserialize($rev->$cls);
			$$packed->{check_requested_authors} = $ids;
			$rev->$cls(MT::Serialize->serialize($packed));
			$save_to->save();
		} else {
			$save_to->check_requested_authors($ids);
			$save_to->save();
		}

		# メール送信のため一次差し替え
		my $backup = $entry->check_requested_authors;
		$entry->check_requested_authors($ids);
		send_notice_mail($app, $entry);
		$entry->check_requested_authors($backup);
	}
}

sub _hdlr_template_param_edit_entry {
	my ($cb, $app, $param, $tmpl) = @_;
	return if unable_plugin();

	my $q = $app->param;
	return unless $q->param('_type') eq 'entry';
	my $user = $app->user;

	# 承認者でなければ処理不要
	return unless is_editor($user);

	# 擬似多重承認のウィジェット表示
	template_param_to_display_cross_check_widget(@_);

	$param->{crr} = $q->param('crr') || '';
	$param->{r} = $q->param('r') || '';

	my $entry = $app->model('entry')->load( $q->param('id') || 0 );

	# リビジョンの場合はPOST情報を復元
	if ( ref($entry) && $param->{crr} && $param->{r} && $param->{error} ) {
		my %param_hash = $app->param_hash;
		my $cols = $entry->column_names || [];
		foreach ( keys %param_hash ) {
			push @$cols, $_ if /^(d_|t_)?customfield_/;
		}
		$param->{$_} = $q->param($_) foreach @$cols;
	}

	# リビジョンを取得
	my $class = $q->param('_type') || 'entry';
	my $r = $param->{r};
	my $rev_obj = load_latest_revision($app, $class, $q->param('id'));

	# プレビューから戻った場合
	my $packed;
	if ( $q->param('reedit') ) {
		$param->{r} = $q->param('pre_r') || '' if defined $q->param('pre_r');
		$param->{keywords} = $q->param('keywords') || '' if $param->{crr};
		$param->{error} ||= $app->param('error') if $app->param('error');
	}
	elsif ( !( defined($param->{error}) && $param->{error} ) ) {
		if ( $param->{r} ) {
			$packed = ref($rev_obj) ? $rev_obj->$class : {};
			$packed = MT::Serialize->unserialize($packed);

			# タグの復元
			$param->{tags} = $$packed->{tags} || ''
				unless $param->{r} eq '0';

			# アイテム情報の復元
			my $packed_objassets = defined($$packed->{objectasset})
				? $$packed->{objectasset}
				: [];
			if ( @$packed_objassets ) {
				my @ids = map { $_->{asset_id} } @$packed_objassets;
				require MT::Asset;
				my @assets = MT::Asset->load({ id => \@ids });
				my $assets = ();
				foreach my $asset (@assets) {
				    my $asset_1 = {
			            asset_id    => $asset->id,
			            asset_name  => $asset->file_name,
				        asset_type  => $asset->class
				    };
				    $asset_1->{asset_thumb} = $asset->thumbnail_url( Width => 100 )
				    	if $asset->class eq 'image';
				    push @{$assets}, $asset_1;
				}
				$param->{asset_loop} = $assets;
			}
		}
	}

	if ( defined($param->{r}) && $param->{r} ) {
		$param->{loaded_revision} = 1;
	}

	# リビジョンの新規作成の場合公開できないようにフラグを落とす
	if ( $param->{crr} && $param->{r} eq '0' ) {
		$param->{status_publish} = 0;
		$param->{check_status} = 0;
	}

	# MT5.1以上の場合crChangeBtnLabelの公開ボタン用文言を修正
	$param->{is_replace} = 1 if MT->version_number >= 5.1 && (
        $q->param('id') && $param->{crr} );

	# 自分の記事の場合は差戻し不可
	my $created_by;
	if ( $param->{crr} ) {
		$created_by = $rev_obj->created_by || 0;
		$created_by = $user->id
			if defined( $q->param('r') ) && $q->param('r') == 0;
	} else {
		$created_by = $app->param('id') ? $entry->created_by : 0;
	}
	$param->{created_by_is_me} = 1
		if $user->id == $created_by || $created_by == 0;

	# 承認者に該当するユーザーを検索
	my $authors = get_checkers();

	my $plugin = $app->component('CheckRelease');

	my $checkers = [];

	# 承認者が存在しない場合はそのメッセージを表示
	unless ( @$authors ) {
		$param->{cr_message} = $plugin->translate("Can't find checkers.");
	}
	else {

		# プレビューなどから戻った場合は選択状態を復元
		if ( defined( $q->{param}->{'checkrelease_checkers[]'} ) ) {
			$checkers = $q->{param}->{'checkrelease_checkers[]'} || [];
		}
		elsif ( defined( $q->param('check_requested_authors') ) ) {
			# 承認者同士の投稿通知とプレビューの場合はこちらでプレビューからの戻り値を処理
			@$checkers = split /\s*,\s*/, $q->param('check_requested_authors');
		}
		elsif ( ref($packed) && defined($$packed->{check_requested_authors}) ) {
			# リビジョンの編集の場合承認者の設定を引き継ぎ
			@$checkers = split /\s*,\s*/, $$packed->{check_requested_authors};
		}
		elsif ( defined($param->{id}) && ref($entry) && !$q->param('error') ) {
			# 記事編集の場合
			@$checkers = split /\s*,\s*/, $entry->check_requested_authors
				if ref($entry) && $entry->check_requested_authors;
		}
		my $object_loop = load_checkers( $checkers );
		$param->{cr_loop} = $object_loop;
	}

	# 差戻し理由のHTML
	my $innerHTML2 = <<'EOD';
		<mt:if name="cr_display_note">
			<textarea type="text" id="check_note" name="check_note" class="text row"><mt:var name="check_note" /></textarea>
		<mt:else>
			<input type="hidden" id="check_note" name="check_note" value="<mt:var name="check_note" escape="html" />" />
			<mt:if name="check_note"><mt:var name="check_note" nl2br="1" /><mt:else><span style="color:#aaa;"><__trans_section component="CheckRelease"><__trans phrase="Nothing is noted."></__trans_section></span></mt:if>
		</mt:if>
EOD

	# 公開ウィジェットのオブジェクトを取得
	my $publishing_node = $tmpl->getElementById('entry-publishing-widget');

	# 差戻し理由のフィールドを作成
	if ( $app->param('id') && (!$app->param('r') || $app->param('r') != 0) ) {
		my $display_check_note = 0;

		# 差し戻し中かつ自分の記事、または承認申請されている記事
		my $node_params;
		$node_params->{id} = 'checkrelease-returnto';
		$node_params->{label} = $plugin->translate('Returnto Reason');
		$param->{cr_display_note} = $param->{created_by_is_me} ? 0 : 1;

		my $block = $tmpl->createElement('app:widget', $node_params);
		$block->innerHTML( $innerHTML2 );
		$tmpl->insertBefore($block, $publishing_node);
	}

	# 承認者選択かどうか
	$param->{select_checker} = get_config_value('select_checker') ? 1 : 0;
	$param->{check_required} = get_config_value('check_required') ? 1 : 0;

	# MT5.1以上の場合、公開ステータスが「公開」の時に指定日公開を選択できないため、
	# リビジョン作成の時に日時指定公開を表示するように修正
	if ( MT->version_number >= 5.1 && $q->param('crr') ) {
		$param->{status_publish} = 0;
	}

	# 承認者同士の承認を行わない場合処理不要
	return 1 unless get_config_value('checker_cross_check');

	# 承認者のHTML
	my $innerHTML = <<'EOT';
		<mt:if name="cr_message">
			<mt:var name="cr_message" />
		<mt:else>
			<p><__trans_section component="CheckRelease"><__trans phrase="You can select your checker."></__trans_section></p>
			<mt:loop name="cr_loop">
			<input type="checkbox" name="checkrelease_checkers[]" class="cr_check_item" value="<mt:var name="id" />"<mt:if name="checked"> checked="checked"</mt:if> onClick="javascript:crChangeBtnLabel();" /> <mt:var name="label" /><br />
			</mt:loop>
		</mt:if>
EOT

	# ボタンのラベル変更スクリプトを追加
	#if ( $param->{created_by_is_me} ) {
	    my $footer    = $tmpl->getElementById('footer_include');
	    my $html_foot = $tmpl->createElement( 'setvarblock',
	        { name => 'html_body_footer', append => 1 } );
	    $html_foot->innerHTML('<script type="text/javascript">' .get_crChangeBtnLabel('editor') .'</script>');
	    $tmpl->insertBefore( $html_foot, $footer );
    #}
	#$innerHTML .= '<script type="text/javascript">' .get_crChangeBtnLabel('editor') .'</script>'
	#	if $param->{created_by_is_me};

	# 他のプラグインから承認者を任意の条件で変更できるようにコールバックを実行
	$app->run_callbacks('cr_template_param.edit_entry', $app, \$innerHTML, \$param)
		or die $app->errstr;

	# 作成者が自分または新規作成の場合
	my $display_auth_widget = 0;

	# 承認者を選択しない場合は処理不要
	if ( get_config_value('select_checker') ) {
		if ( !$q->param('id') ) {
			++$display_auth_widget;
		} elsif ( $param->{crr} && $param->{r} == 0 ) {
			++$display_auth_widget;
		} elsif ( $param->{created_by_is_me} ) {
			++$display_auth_widget;
		}
	}

	if ( $display_auth_widget ) {
		# MTタグでHTMLを生成
		my $node_params;
		$node_params->{id} = 'checkrelease-checkeres';
		$node_params->{label} = $plugin->translate('Select Checkers');
		my $block = $tmpl->createElement('app:widget', $node_params);

		$block->innerHTML( $innerHTML ) ;

		#公開ウィジェットの下に挿入
		$tmpl->insertBefore($block, $publishing_node);
	}

}

sub _hdlr_cms_post_save {
	my ($cb, $app, $obj, $obj_org) = @_;
	my $q = $app->param;

	return if unable_plugin();

	my %param_hash = $app->param_hash;

	my $plugin = $app->component('CheckRelease');
	my $blog_id = $q->param('blog_id') || 0;
	my $config = get_config_hash();

	my $user = $app->user;

	# 投稿者が承認者でない場合処理不要
	return unless is_editor($user);

	# 擬似多重承認のメール送信
	cms_post_save_to_send_cross_check_mail(@_) unless $app->param('crr');

	# 一括編集の場合
	my $mode = $app->param('__mode') || '';
	if ( $mode eq 'save_entries' ) {
		my $rev_obj = load_latest_revision($app, $obj->class, $obj->id);
		my $datasource = $obj->datasource;
		my $packed_obj = MT::Serialize->unserialize( $rev_obj->$datasource );
		$$packed_obj->{created_by} = $obj->author_id;
		$rev_obj->$datasource( MT::Serialize->serialize($packed_obj) );
		$rev_obj->save()
			or return $cb->erorr( $rev_obj->errstr );
		return 1;
	}

	# 承認状態を取得
	my $check_status = $q->param('check_status') || 0;

	my $blog = $app->model('blog')->load($blog_id);

	# 承認者同士の承認設定を取得
	my $cross_check = ( defined($config->{checker_cross_check})
		&& $config->{checker_cross_check} ) ? 1 : 0;

	# 公開又は公開日指定の場合承認情報をリセット
	if ( $obj->status == MT::Entry::RELEASE() || $obj->status == MT::Entry::FUTURE() ) {
		$obj->check_status( APPROVAL() );
		$obj->check_requested_authors( '' );
		$obj->checked_on(
			format_ts( '%Y-%m-%d %H:%M:%S',
				epoch2ts($blog, time()), $blog, 'en', 0) );
		$obj->save()
			or return $cb->error( $obj->errstr );
		return 1;
	}
	elsif ( $check_status == RETURN() ) {
		# 差戻しの場合
		$obj->check_status( RETURN() );
		$obj->check_note( $q->param('check_note') );
		$obj->checked_on(
			format_ts( '%Y-%m-%d %H:%M:%S',
				epoch2ts($blog, time()), $blog, 'en', 0) );
		$obj->checked_by( $user->id );
		$obj->save()
			or return $cb->error( $obj->errstr );

		# 承認者、投稿者に差戻し通知。プレビューから保存されたときに差し戻し通知が再送されないように条件文を追加
		send_returnto_mail( $app, $obj ) unless $user->id eq $obj->created_by;
		return;
	}
	else {
		if ( ref($obj_org) && $obj_org->check_status() == SUBSCRIBED() ) {
			$obj->check_status( SUBSCRIBED() );
		}
	}
	$obj->save()
		or return $cb->error( $obj->errstr );

	# 承認者同士の承認が必要な場合
	if ( $cross_check && is_editor($obj->author) && $obj->created_by == $user->id ) {

		# 承認者選択設定の場合
		my $select_checker = ( defined($config->{select_checker})
			&& $config->{select_checker} ) ? 1 : 0;

		my $terms;
		if ( $select_checker ) {
			$terms->{id} = $q->{param}->{'checkrelease_checkers[]'} || [];
		}

		my $checkers = get_checkers( defined($terms->{id}) ? $terms : undef );
		my @ids = map { $_->id } @$checkers;
		my $ids = @ids ? join(',', @ids) : '';

		if ( $ids ) {
			# 下書き保存
			$obj->status( MT::Entry::HOLD() );

			if (my $save_draft = $q->param('save_draft') == 1) {
				$ids = undef;
			}

			# 下書きした場合は承認中にしない
			if ( $ids ) {
				$obj->check_requested_authors( $ids );
				$obj->check_status( SUBSCRIBED() );
			}

			$obj->save()
				or return $cb->error( $obj->errstr );

			# 投稿通知
			send_notice_mail( $app, $obj ) if $ids;
		}

		# 編集不可の場合記事一覧へリダイレクト
		if ( defined($config->{lock_entry}) && $config->{lock_entry}
			&& $obj->check_status == SUBSCRIBED() ) {
			my $arg;
			$arg->{_type} = $app->param('_type');
			$arg->{blog_id} = $blog_id;
			my $goto = list_mode($obj->datasource);
			my $list_uri = $app->mt_uri( mode => $goto, args => $arg );
			return do_redirect($app, $list_uri);
		}
	}
}

sub list_mode {
	my $class = shift;
	return MT->version_number >= 5.1 ? 'list' : "list_$class";
}

sub send_returnto_mail {
	my ($app, $obj, $author) = @_;
	my $user = $app->user;
	my $plugin = $app->component('CheckRelease');

	# 承認者情報をemailでまとめる
	my $terms;
	$terms->{id} = [ split /\s*,\s*/, $obj->check_requested_authors ];
	my $checkers = get_checkers($terms);
	my %emails;
	foreach ( @$checkers ) {
		next if defined $emails{$_->email};
		$emails{ $_->email } = $_;
	}

	# 投稿者に差し戻し通知
	$author = $obj->author unless ref $author;
	my $title = $plugin->translate("'[_1]'(ID:[_2])' was returned.", $obj->title, $obj->id);
	my $edit_url = check_url($app, $obj, 'view');

	# 差し戻しメールテンプレートから本文を生成
	my $param = {};
	$param->{editor} = $user;
	$param->{entry} = $obj;
	$param->{edit_url} = $edit_url;
	$param->{blog} = $obj->blog;
	$param->{check_note} = $app->param('check_note');
	my $body = $app->build_email('return_to_entry_email', $param);

	# メールヘッダーの取得
	my $from = $app->config->EmailAddressMain || '';
	die $plugin->translate('EmailAddressMain is required.')
		unless $from;
	my %head = mail_head($app, $title, $from);
	return unless %head;

	# メール送信
	$emails{ $author->email } = 1;

	require MT::Mail;
	foreach ( keys %emails ) {
		$head{To} = $_;
		MT::Mail->send( \%head, $body )
			or $app->log( MT::Mail->errstr );
	}

}


sub load_meta_field_basename {
	my ($blog_id, $class) = @_;
	require CustomFields::Field;
	my $terms;
	$terms->{obj_type} = $class;
	$terms->{blog_id} = [0, $blog_id];
	my @fields = CustomFields::Field->load($terms) || ();
	return [] unless @fields;
	my @names = map { $_->basename } @fields;
	return @names ? \@names : [];
}

# 差し戻しカスタムフィールドの情報を返す(不要？)
sub get_return_to {
	my $app = MT::App::CMS->instance;
	my $q = $app->param;
	my $plugin = $app->component('CheckRelease');
	my $terms;
	$terms->{blog_id} = $q->param('blog_id') || 0;
	$terms->{type} = 'checkrelease';
	require CustomFields::Field;
	my @fields = CustomFields::Field->load($terms);
	my @field_basename;
	push @field_basename, {
		name => $_->name,
		basename => $_->basename,
		fieldname => 'customfield_' .$_->basename
	} foreach @fields;
	\@field_basename;
}


sub _hdlr_template_source_entry_table {
	my ($eh, $app, $tmpl) = @_;

	return $$tmpl if unable_plugin();

	my $user = $app->user;
	return $$tmpl unless is_editor($user);

	# 記事一覧以外の画面では処理しない
	my $mode = $app->param('__mode') || '';
	return $$tmpl unless $mode eq 'list_entry';
	return $$tmpl
		if $mode eq 'list_entry'
			&& ($app->param('filter')
				&& $app->param('filter') eq 'power_edit');

	# 記事のステータスアイコン部分に承認系アイコンを表示
	my $icon_old = '<mt:if name="status_spam">\s+?<a href=.+?<mt:if[^>]+?>.+?</mt:if>.+?<mt:if[^>]+?>.+?</mt:if>.+?</a>\s*?</mt:if>';
	my $icon_new = '<mt:if name="check_status_subscribed"><img src="<mt:StaticWebPath />images/status_icons/warning.gif" width="9" height="9" /></mt:if><mt:if name="check_status_return"><img src="<mt:StaticWebPath />images/status_icons/share.gif" width="9" height="9" /></mt:if>';

	$$tmpl =~ s!($icon_old)!$1\n$icon_new!mgs;

	# リビジョン作成リンクを表示
	my $rev_old = '<th class="category">';
	my $rev_new = '<th style="width:12%;"><__trans_section component="CheckRelease"><__trans phrase="Revision" /></__trans_section></th>';
	$$tmpl =~ s!$rev_old!$rev_new\n$rev_old!m;

	my $old = '<td class="category">';
	my $new = '<td>'.list_html_tmpl().'</td>';

	$$tmpl =~ s!$old!$new\n$old!m;
	$$tmpl;
}

sub _hdlr_cms_pre_preview {
	my ($cb, $app, $entry, $data) = @_;
	return if unable_plugin();

	my $q = $app->param;
	return unless $q->param('_type') eq 'entry';
	my $user = $app->user;

	return unless is_editor($user);

	# 承認者選択のパラメータがあればhiddenに追加
	if ( defined $q->{param}->{'checkrelease_checkers[]'} ) {
		my $checkers = $q->{param}->{'checkrelease_checkers[]'} || [];
		if ( @$checkers ) {
			foreach ( @$checkers ) {
				push @$data, {
					data_name => 'checkrelease_checkers[]',
					data_value => $_
				};
			}
		}
		else {
			# 未選択状態の場合
			push @$data, {
				data_name => 'checkrelease_checkers[]',
				data_value => ''
			};
		}
	}

	foreach ( qw/r crr/ ) {
		my $key = $_;
		if ( $_ eq 'r' ) {
			$key = 'pre_r';
		}
        my $value = $app->param($_);
		push @$data, {
			data_name => $key,
			data_value => $value,
		};
	}
}

sub _hdlr_template_source_list_entry {
	my ($cb, $app, $tmpl) = @_;
	return $$tmpl if unable_plugin();

	my $user = $app->user;
	my $blog_id = $app->param('blog_id');
	return unless is_editor($user);

	# 記事一覧以外の画面では処理しない
	my $mode = $app->param('__mode') || '';
	my $type = $app->param('_type') || 'entry';
	return $$tmpl unless $mode eq list_mode($type);
	return $$tmpl
		if $mode eq list_mode($type)
			&& ($app->param('filter')
				&& $app->param('filter') eq 'power_edit');

	my $scope = "blog:$blog_id";
	my $lock_entry = get_config_value('lock_entry', $scope);

	# 記事ロック中に編集画面等に直接アクセスした場合のエラーメッセージ
	if ( $lock_entry ) {
		my $status_deleted = MT->version_number < 5.1
			? '<mt:if name="saved_deleted">'
			: '<mt:if name="request\.saved_deleted">';
		my $status_lock_entry = q{
    <mt:if name="lock_entry">
        <mtapp:statusmsg
            id="lock-entry"
            class="error"
            rebuild=0>
            <__trans_section component="CheckRelease">
            <mt:if name="object_type" eq="entry">
                <__trans phrase="The entry is subscribed and locked.">
            <mt:else>
                <__trans phrase="The page is subscribed and locked.">
            </mt:if>
        </mtapp:statusmsg>
    </mt:if>
		};
		$$tmpl =~ s!($status_deleted)!$status_lock_entry\n$1!mi;
	}
}

sub _hdlr_template_param_list_entry {
	my ($cb, $app, $param, $tmpl) = @_;
	return if unable_plugin();

	my $plugin = $app->component('CheckRelease');

	# 承認者でなければ処理不要
	my $user = $app->user;
	return unless is_editor($user);

	# 記事一覧以外の画面では処理しない
	my $mode = $app->param('__mode') || '';
	my $type = $app->param('_type') || 'entry';
	return $$tmpl unless $mode eq list_mode($type);
	return $$tmpl
		if $mode eq list_mode($type)
			&& (defined($app->param('filter'))
				&& $app->param('filter') eq 'power_edit');

	my $class = $app->param('_type') || 'entry';

	my $config = get_config_hash();

	# 記事ロックの設定を取得
	my $lock_entry = ( defined($config->{lock_entry})
		&& $config->{lock_entry} ) ? 1 : 0;

	if ( MT->version_number < 5.1 ) {
		# 承認者同士の承認設定を取得
		my $cross_check = ( defined($config->{checker_cross_check})
			&& $config->{checker_cross_check} ) ? 1 : 0;

		# 承認中の記事のアイコンの表示切替
		foreach my $entry ( @{ $param->{entry_table}[0]{object_loop} } ) {
			$entry = get_entry_status($app, $user, $entry, $class, $lock_entry, $cross_check);
		}
	}

	# 記事ロックフラグがパラメータで渡されたらonにする
	$param->{lock_entry} = 1 if $app->param('lock_entry');
}

sub get_entry_status {
	my ($app, $user, $entry, $class, $cross_check, $lock_entry, $obj) = @_;

	if ( defined($obj) && ref($obj) ) {
		my $cols = $obj->column_names;
		foreach my $c ( @$cols ) {
			$entry->{$c} = $obj->$c;
		}
	}

	if ( defined($entry->{check_status}) && $entry->{check_status} ) {
		$entry->{has_edit_access} = 1;
		delete $entry->{status_Draft};
		if ( $entry->{check_status} == RETURN() ) {
			$entry->{check_status_return} = 1;
		} elsif ( $entry->{check_status} == SUBSCRIBED() ) {
			$entry->{check_status_subscribed} = 1;
			# 新規作成でかつ承認申請中の記事はリンクを除去
			if ( $cross_check
				&& $lock_entry
				&& $entry->{created_by} == $user->id ) {
				$entry->{has_edit_access} = 0;
			}
		}
	}

	# 最新リビジョンの取得
	my $rev_obj = load_latest_revision($app, $class, $entry->{id});
	my $rev_number = 0;
	if ( ref($rev_obj) ) {
		$rev_number = $rev_obj->rev_number;

		# リビジョンの公開状態を取得
		my $rev_entry = MT::Serialize->unserialize($rev_obj->$class);
		# リビジョンの承認状態を取得
		my $check_status = $$rev_entry->{check_status} || 0;
		$entry->{rev_has_edit_access} = 1;

        my $status = $$rev_entry->{status} || 0;
        my $is_approved = $status == MT::Entry::RELEASE()
            || $status == MT::Entry::FUTURE()
            || $status == MT::Entry::JUNK()
            || $status == 6 ? 1 : 0; # 6=MT::Entry::UNPUBLISH() compatibility under 6

		if ( !$is_approved && $check_status == SUBSCRIBED() ) {
			$entry->{rev_check_status_subscribed} = 1;

			# 新規作成でかつ承認申請中の記事はリンクを除去
			if ( $cross_check
				&& $lock_entry
				&& $rev_obj->created_by == $user->id ) {
				$entry->{rev_has_edit_access} = 0;
			}
		} elsif ( !$is_approved && $check_status == RETURN() ) {
			$entry->{rev_check_status_return} = 1;
		} elsif ( $$rev_entry->{status} == MT::Entry::HOLD() ) {
			$entry->{rev_status_draft} = 1;
		} elsif ( $$rev_entry->{status} == MT::Entry::RELEASE() ) {
			$entry->{rev_status_publish} = 1;
		} elsif ( $$rev_entry->{status} == MT::Entry::FUTURE() ) {
			$entry->{rev_status_future} = 1;
		} elsif ( $$rev_entry->{status} == 6 ) { # 6=MT::Entry::UNPUBLISH() compatibility under 6s
            $entry->{rev_status_unpublish} = 1;
        }
		if ( $rev_number == $entry->{current_revision} ) {
			$entry->{is_latest} = 1;
		}
	}
	if ( $entry->{status} == MT::Entry::HOLD() ) {
		$entry->{rev_has_edit_access} = 0;
	}
	$entry->{rev_number} = $rev_number;

	# システム管理者は全て編集可能
	if ( $cross_check && $user->is_superuser ) {
		$entry->{rev_has_edit_access} = 1
			if $entry->{status} != MT::Entry::HOLD();
	}
	$entry;
}

sub _hdlr_cms_pre_save_entry {
	my ($cb, $app, $obj, $obj_orig) = @_;

	# 承認者で無ければ処理不要
	my $user = $app->user;
	return 1 unless is_editor($user);

	# 新規作成の場合処理不要
	return 1 unless $app->param('id');

	# 一括編集の場合
	my $mode = $app->param('__mode') || '';
	if ( $mode eq 'save_entries' ) {
		my $rev_obj = load_latest_revision($app, $obj->class, $obj->id);
		my $datasource = $obj->datasource;
		my $packed_obj = MT::Serialize->unserialize( $rev_obj->$datasource );
		$$packed_obj->{created_by} = $obj->author_id;
		$rev_obj->$datasource( MT::Serialize->serialize($packed_obj) );
		$rev_obj->save()
			or return $cb->erorr( $rev_obj->errstr );
		return 1;
	}

	# リビジョンでなければ処理不要
	my $q = $app->param;
	my $crr = $q->param('crr') || 0;
	return 1 unless $crr;

	# 最新リビジョンを取得
    my $blog_id = $q->param('blog_id');
    my $type = $q->param('_type') || 'entry';

	my $rev_obj = load_latest_revision($app, $obj->datasource, $obj->id);
	my $r = $q->param('r') || 0;

	my $orig_check_status = 0;
	if ( ref($rev_obj) ) {
		my $orig_packed = MT::Serialize->unserialize( $rev_obj->$type );
		$orig_check_status = defined($$orig_packed->{check_status})
			? $$orig_packed->{check_status}
			: 0;
	}

	# 公開日指定の場合statusをFUTUREに変更
	my $status = $app->param('status');
	$status = MT::Entry::FUTURE() if $app->param('scheduled');

	# 差し替えの場合処理不要
	if ( $status == MT::Entry::RELEASE() ) {

		# ライターの記事を差替した場合にライターが編集できなくなるためcreated_byを上書き
		$obj->created_by( $obj_orig->created_by );
		$obj->current_revision( $rev_obj->rev_number );
		$obj->check_requested_authors( '' );
		$obj->check_status( APPROVAL() );
		$obj->check_note( '' );
		return 1;
	}


	# 新規作成又は古いリヴィジョンから下書きまたは差戻した場合、最新のリビジョンを作成
	my $new_revision = ($r == 0) ? 1 : 0;
	if ( $new_revision || (ref($rev_obj) && $r && $r < $rev_obj->rev_number) ) {
		$obj->{changed_revisioned_cols} = [];
		my $latest_rev_number = ref($rev_obj) ? $rev_obj->rev_number : 0;
		my $rev_num = ($r == 0) ? $latest_rev_number : 0;
		local $obj->{column_values}->{current_revision} = $rev_num;
		$r = $obj->save_revision();
		$rev_obj = load_latest_revision($app, $obj->datasource, $obj->id);
	}
	else {
		# 下書き保存の場合、最新リビジョンの承認状態を引き継ぎ
		$obj->check_status( $orig_check_status )
			if $obj->status == MT::Entry::HOLD();
	}

	# 承認者同士の承認が必要かどうか
	my $config = get_config_hash();
	my $cross_check = ( defined($config->{checker_cross_check})
		&& $config->{checker_cross_check} ) ? 1 : 0;

	my $blog = $obj->blog;

	my $ids = '';
	my $check_status = $q->param('check_status') || 0;
	$check_status = 0 if $check_status != 0 && $r == 0;

	# 承認者同士の承認が必要な場合
	if ( $cross_check
		&& (
			( !$obj->created_by || $obj->created_by == $user->id )
			|| $rev_obj->created_by == $user->id
			) ) {
		# 承認者選択設定の場合
		my $select_checker = ( defined($config->{select_checker})
			&& $config->{select_checker} ) ? 1 : 0;

		my $terms;
		if ( $select_checker ) {
			$terms->{id} = $q->{param}->{'checkrelease_checkers[]'} || [];
		}
		my $checkers = [];

		# 承認申請情報を保存
		if ( defined($terms->{id}) ) {
			$checkers = $terms->{id} || [];
		}
		else {
			my $checker_objs = get_checkers() || ();
			$checkers = [ map { $_->id } @$checker_objs ];
		}
		$ids = join ',', @$checkers;

		my $save_draft = $app->param('save_draft');
		$ids = undef if $save_draft == 1;
		if ( defined $ids ) {
			$obj->status( MT::Entry::HOLD() );
			$obj->check_status( SUBSCRIBED() ) if $ids;
			$obj->check_requested_authors( $ids );
		}
	}

	# 指定日公開の場合
	if ( $check_status != SUBSCRIBED() && $status == MT::Entry::FUTURE() ) {
		$obj->check_status( APPROVAL() );
		$obj->check_requested_authors( '' );
		$obj->checked_on(
			format_ts( '%Y-%m-%d %H:%M:%S',
				epoch2ts($blog, time()), $blog, 'en', 0) );
	}
	else {

		# 差戻しの場合
		if ( $check_status == RETURN() ) {
			$obj->check_status( RETURN() );
			$obj->checked_by( $user->id );
			$obj->checked_on(
				format_ts( '%Y-%m-%d %H:%M:%S',
					epoch2ts($blog, time()), $blog, 'en', 0) );
		}
		$obj->check_note( $q->param('check_note') );

		# リビジョン新規作成以外
		if ( !$new_revision ) {
			my $serialized_obj = $rev_obj->entry;
			my $packed_rev_obj = MT::Serialize->unserialize($serialized_obj);
			my $author = MT::Author->load($rev_obj->created_by);
			# ライターからの承認か承認者同士の承認処理か、
			# および承認者同士でリビジョン新規作成か編集か、
			# で下書き保存するときに承認者設定を引き継ぐ処理の分岐
			if (is_editor($author)
				&& ($app->param('r') == 0 || $rev_obj->created_by == $user->id) ) {
				$obj->check_requested_authors( $ids);
			} else {
				$obj->check_requested_authors( $$packed_rev_obj->{check_requested_authors} );
			}
			$obj->created_by( $$packed_rev_obj->{created_by} );
		}
	}

    # リビジョンハッシュ化前のコールバック(MoreCustomFields対策)
    $app->run_callbacks('checkrelease_pre_pack_revision.entry', $app, $obj, $obj_orig);

	# リビジョンの取得
	my $packed_obj = $obj->pack_revision();
	$packed_obj->{status} = $status;
	$packed_obj->{created_by} = $user->id if $new_revision;

	# カスタムフィールドの値を上書き
	my $cf_meta = cf_meta($app, $obj);
    foreach my $key (keys %$cf_meta) {
    	$packed_obj->{"field.$key"} = $cf_meta->{$key};
    }

    my $class = $app->model($type)
    	or return $cb->error( $app->errtrans('Invalid parameter') );
    my $cat_class = $app->model( $class->container_type );

    # アイテムとの関連を保存
    my @packed_objassets;
    require MT::Asset;
    require MT::ObjectAsset;
    my $include_asset_ids = $app->param('include_asset_ids');
    my @asset_ids = split(',', $include_asset_ids);
    my $obj_assets = ();
    my @obj_assets = MT::ObjectAsset->load({ object_ds => 'entry', object_id => $obj->id });
    foreach my $obj_asset (@obj_assets) {
        my $asset_id = $obj_asset->asset_id;
        $obj_assets->{$asset_id} = 1;
    }
    my $seen = ();
    foreach my $asset_id (@asset_ids) {
        my $obj_asset = MT::ObjectAsset->load({ asset_id => $asset_id, object_ds => 'entry', object_id => $obj->id });
        unless ($obj_asset) {
            my $obj_asset = new MT::ObjectAsset;
            $obj_asset->blog_id($blog_id);
            $obj_asset->asset_id($asset_id);
            $obj_asset->object_ds('entry');
            $obj_asset->object_id($obj->id);
            my $packed = pack_revision($obj_asset);
            push @packed_objassets, $packed;
        }
    }
    $packed_obj->{objectasset} = \@packed_objassets;

    # カテゴリの関連を保存
    my ( $cat_id, @add_cat ) = split /\s*,\s*/,
      ( $app->param('category_ids') || '' );

    my @packed_places;
    require MT::Placement;
    my $place =
      MT::Placement->load( { entry_id => $obj->id, is_primary => 1 } );
    unless ($place) {
        $place = MT::Placement->new;
        $place->entry_id( $obj->id );
        $place->blog_id( $obj->blog_id );
        $place->is_primary(1);
    }
    $place->category_id($cat_id);
    push @packed_places, pack_revision($place);

    # save secondary placements...
    for my $cat_id (@add_cat) {
        my $cat = $cat_class->load($cat_id);

        # blog_id sanity check
        next if !$cat || $cat->blog_id != $obj->blog_id;

        my $place = MT::Placement->new;
        $place->entry_id( $obj->id );
        $place->blog_id( $obj->blog_id );
        $place->is_primary(0);
        $place->category_id($cat_id);
        push @packed_places, pack_revision($place);
    }
    $packed_obj->{placement} = \@packed_places;

	# タグを保存
	$packed_obj->{tags} = $app->param('tags') || '';

	my $changed_cols = $obj->{changed_revisioned_cols} || [];
	my $datasource = $obj->datasource;
	my $obj_id = $datasource .'_id';
	$obj->gather_changed_cols($obj_orig, $app);
	$rev_obj->$datasource( MT::Serialize->serialize(\$packed_obj) );
	$rev_obj->changed( join ',', @$changed_cols );
    $rev_obj->description( $app->param('revision-note') );
	$rev_obj->save()
		or return $cb->erorr( $rev_obj->errstr );

	# 日時指定公開の場合は日時指定キューを更新
	update_future_que($app, $rev_obj, $datasource);

	# 承認者同士の承認が必要な場合
	if ( $user->id == $rev_obj->created_by
		&& $cross_check && $status == MT::Entry::HOLD()
		&& ( $check_status == 0
		|| $check_status == SUBSCRIBED() ) ) {

		$obj->created_by( $rev_obj->created_by );

		send_notice_mail( $app, $obj, $user ) if $ids;
	}
	elsif ( $check_status == RETURN() ) {
		# 承認者、投稿者に差戻し通知
		$q->param('r', $rev_obj->rev_number);
		my $rev_creator = $app->model('author')->load( $rev_obj->created_by ) || undef;
		send_returnto_mail( $app, $obj, $rev_creator );
	}

	# 自動保存のセッション情報を削除
    my $sess_obj = $app->autosave_session_obj;
    $sess_obj->remove if $sess_obj;

	# リダイレクト
	my $args;
	$args->{r} = $rev_obj->rev_number;
	foreach ( qw/blog_id id _type crr/ ) {
		$args->{$_} = $q->param($_);
	}
	$args->{saved_changes} = 1;

	# 擬似多重承認のメール送信
	cms_post_save_to_send_cross_check_mail(@_) if $app->param('crr');

	# 記事編集禁止の場合
	my $lock_entry = ( defined($config->{lock_entry})
		&& $config->{lock_entry} && $obj->check_status == SUBSCRIBED() ) ? 1 : 0;
	$lock_entry = 0
		unless $lock_entry
			&& $packed_obj->{created_by} == $user->id;

	my $url = $app->mt_uri(
		mode => $lock_entry ? list_mode($type) : 'view',
		args => $args );
	return do_redirect($app, $url);
}

sub do_redirect {
	my ($app, $url) = @_;
	my $q = $app->param;
	my $qr = ref $q;
	if ( $ENV{MOD_PERL} ) {
	    $app->{apache}->header_out( Location => $url );
	    $app->response_code( Apache::Constants::REDIRECT() );
	    $app->send_http_header;
	}
	elsif ( $qr =~ /::PSGI/i ) {
		$app->redirect($url);
		return 0;
	}
	else {
	    $app->print(
	        $q->redirect( -uri => $url, %{ $app->{cgi_headers} } ) );
	}
	exit;
}

sub pack_revision {
	my $obj = shift;
	my $class = ref $obj || $obj;
	my $values = $obj->column_values;
	my $meta_values = $obj->meta;
	foreach my $key (keys %$meta_values) {
		next if $key eq 'current_version';
		$values->{$key} = $meta_values->{$key};
	}
	return $values;
}

sub _hdlr_pre_run {
	my ($cb, $app) = @_;
	return 1 if unable_plugin();
	return 1 unless ref($app) eq 'MT::App::CMS';
	my $q = $app->param;
	my $mode = $app->mode;

	return 1 unless $mode eq 'view';
	my $class = $q->param('_type') || '';
	return 1 unless $class eq 'entry' || $class eq 'page';
	my $blog_id = $q->param('blog_id') || 0;
	return 1 unless $blog_id;

	# 承認者でなければ処理不要
	my $user = $app->user;
	return 1 unless is_editor($user);

	# 記事ロック設定かどうか
	my $scope = "blog:$blog_id";
	my $lock_entry = get_config_value('lock_entry', $scope);
	my $cross_check = get_config_value('checker_cross_check', $scope);

	my $plugin = $app->component('CheckRelease');
	my $id = $q->param('id');
	my $r = defined($q->param('r')) ? $q->param('r') : '';
	my $crr = $q->param('crr') || '';

	# 記事ロック設定の場合
	if ( $lock_entry && $mode eq 'view' && !$user->is_superuser ) {
		my $entry = {};
		my $rev_obj = load_latest_revision($app, $class, $id);
		if ( $r && $r != 0 && $crr && ref($rev_obj) ) {
			my $packed = MT::Serialize->unserialize( $rev_obj->$class ) || {};
			$entry = $$packed
				if $$packed->{created_by} == $user->id
					&& defined($$packed->{check_status});
		}
		unless ( %$entry ) {
			my $pkg = $app->model($class);
			my $obj = $id ? $pkg->load($id) : $pkg->new;
			my $columns = $obj->column_names;
			%$entry = map { $_ => $obj->$_ } @$columns;
		}
		# 承認申請中の場合記事一覧へリダイレクト
		my $redirect = 0;

		# 作者が自分の場合はリダイレクト判定
		if ( $cross_check && defined($entry->{created_by}) && $entry->{created_by} eq $user->id ) {
			# 承認申請中の場合
			if ( $entry->{check_status} == SUBSCRIBED() ) {
				++$redirect;
			}
			elsif ( $crr && $r != 0 &&
				( $entry->{status} == MT::Entry::FUTURE()
					|| $entry->{status} == MT::Entry::RELEASE() ) ) {
				# リビジョンまたは記事が、公開準備中または公開中の場合
				++$redirect;
			}
		}
		if ( $redirect ) {
			my $params;
			$params->{blog_id} = $blog_id;
			$params->{_type} = $class;
			$params->{lock_entry} = 1;
			my $goto = list_mode($class);
			my $uri = $app->mt_uri( mode => $goto, args => $params);
			return $app->redirect($uri);
		}
	}

	# プレビューでない場合
	unless ( $q->param('reedit') ) {
		# リビジョンを取得
		my $rev_obj = load_latest_revision($app, $class, $q->param('id'));
		my $packed = ref($rev_obj) ? $rev_obj->$class : {};

		$packed = MT::Serialize->unserialize($packed);

		# アイテムのパラメータを擬似復元
		my $packed_objassets = defined($$packed->{objectasset})
			? $$packed->{objectasset}
			: [];
		if ( @$packed_objassets ) {
			$q->param('include_asset_ids',
				join(',', map { $_->{asset_id} } @$packed_objassets) );
		}

		# カテゴリのパラメータを擬似復元
		my $packed_placement = defined($$packed->{placement})
			? $$packed->{placement}
			: [];
		if ( @$packed_placement && $q->param('crr') ) {
			my @cat_ids = map { $_->{category_id} } grep { defined $_->{category_id} } @$packed_placement;
			$q->param('category_ids',  join(',', @cat_ids )) if @cat_ids;
		}
	}
}

sub _hdlr_cms_save_filter_entry {
	my ($cb, $app) = @_;
	return 1 if unable_plugin();

	my $user = $app->user;
	return 1 unless is_editor($user);
	return 1 if $user->is_superuser;

	my $q = $app->param;
	my $r = $q->param('r') || 0;
	my $crr = $q->param('crr') || 0;

	my $config = get_config_hash();

	# 承認者同士の承認が必要かどうか
	my $cross_check = ( defined($config->{checker_cross_check})
		&& $config->{checker_cross_check} ) ? 1 : 0;

	# 承認者の選択が可能かどうか
	my $select_checker = ( defined($config->{select_checker})
		&& $config->{select_checker} ) ? 1 : 0;

    # 承認者同士の承認を認めない場合はEditor側でこれ以上、処理はない
    return 1 if !$cross_check;

    # 承認者を選択しなくて良い場合もこれ以上処理は不要
    return 1 if !$select_checker;

    # 承認者同士の承認ができ、かつ承認者の選択が必要な場合にここからの処理を行う
	my $plugin = $app->component('CheckRelease');

	# 新規作成の場合
	my $status = $q->param('status');

	# 公開処理の場合承認者の選択不要
	return 1 if ( $status == MT::Entry::RELEASE()
			|| $status == MT::Entry::FUTURE() );

	# 作者が自分で無ければ処理不要
    my $id = $q->param('id') || '';
	my $author;
	my $obj;
	my $class = $q->param('_type') || 'entry';
	if ( $crr ) {
		$author = $user;
		unless ( $r == 0 ) {
			$obj = load_latest_revision($app, $class, $id);
			$author = $app->model('author')->load( $obj->created_by );
		}
	}
	else {
		$author = $user;
		if ( $id ) {
			$obj = $app->model($class)->load( $id );
			$author = $obj->author;
		}
	}
	return 1 unless $author->id == $user->id;

	my $save_draft = $app->param('save_draft');

	if ( defined($config->{select_checker}) && $config->{select_checker} && $save_draft == 0 ) {
		my $id = $q->{param}->{'checkrelease_checkers[]'} || [];
		if ( !@$id && (
			!defined($config->{check_required})
			|| (
				defined($config->{check_required})
				&& $config->{check_required}
				)
			) ) {
			$app->error($plugin->translate('Auchenticator is required.'));
			return $cb->error( $plugin->translate('Auchenticator is required.') );
		}
	}
	1;
}

sub _hdlr_pre_build {
	my ($cb) = @_;
	return if unable_plugin();
	require MT::App::CMS;
	my $app = MT::App::CMS->instance;
	my $user = $app->user;
	return unless is_editor($user);
	my $type = $app->param('_type') || '';
	return unless 'entry';
	my @id = $app->param('id');
	return unless @id;
	my $terms;
	$terms->{id} = \@id;
	my @entries = $app->model('entry')->load( $terms );
	foreach ( @entries ) {
		$_->check_status( APPROVAL() );
		$_->check_requested_authors( '' );
		$_->save();
	}
}


sub _hdlr_cms_filtered_list_param_entry {
	my ($cb, $app, $res, $objs) = @_;
	return if unable_plugin();

	my %objs = map { $_->id => $_ } @$objs;
	my $objects = $res->{objects} || [];
	return unless @$objects;

	my $plugin = $app->component('CheckRelease');

	# loop objects, check status and insert status icon
	foreach my $row ( @$objects ) {
		my $entry = $objs{ $row->[0] };
		my $rev_obj = load_latest_revision($app, 'entry', $entry);
		my $is_rev_obj = ref($rev_obj) ? 1 : 0;

		# Get statuses
		my $status = $entry->status;
		my $rev_status;
		my $rev_check_status;
		if ( ref($rev_obj) ) {
			my $packed = $rev_obj->entry;
			$packed = MT::Serialize->unserialize($packed);
			$rev_status = $$packed->{status};
			$rev_check_status = $$packed->{checked_status};
		}
	}
}

sub _hdlr_cms_post_save_bulk_save_entries {
	my ($cb, $app, $objects) = @_;
	return if unable_plugin();
	return unless $app->mode eq 'save_entries';
	return unless @$objects;
	my $class = $app->param('_type') || '';
	return unless $class;
	foreach my $obj_hash ( @$objects ) {
		my $obj = $obj_hash->{current};
		# change created_by
		$obj->created_by( $obj->author_id );
		$obj->save()
			or $app->log( $app->translate(
				"Saving entry '[_1]' failed: [_2]", $obj->title,
				$obj->errstr
			) );
		# if there is new revision, change it too
		my $rev_obj = load_latest_revision($app, $class, $obj->id);
		next unless $rev_obj->rev_number > $obj->current_revision;
		my $packed = $rev_obj->$class;
		$packed = MT::Serialize->unserialize($packed);
		next unless ref $packed;
		$$packed->{created_by} = $obj->author_id;
		$$packed->{author_id} = $obj->author_id;
		$rev_obj->$class( MT::Serialize->serialize($packed) );
		$rev_obj->created_by( $obj->author_id );
		$rev_obj->save()
			or $app->log( $app->translate(
				"Saving entry '[_1]' failed: [_2]", $obj->title,
				$rev_obj->errstr
			) );
	}
}
1;
